﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;

namespace ELTE.Basics.University
{
    /// <summary>
    /// Egyetemi kurzus típusa.
    /// </summary>
    class Course
    {
        private List<Student> _Students; // kurzus hallgatói

        /// <summary>
        /// Kurzus nevének lekrédezése.
        /// </summary>
        public String Name { get; private set; }

        /// <summary>
        /// Kurzus oktatójának lekérdezése.
        /// </summary>
        public Teacher Teacher { get; private set; }

        /// <summary>
        /// Hallgatók számának lekérdezése.
        /// </summary>
        public Int32 StudentCount { get { return _Students.Count; } }

        /// <summary>
        /// Kurzus létrehozása.
        /// </summary>
        /// <param name="teacher">Oktató.</param>
        /// <param name="name">Kurzus neve.</param>
        public Course(Teacher teacher, String name)
        {
            Name = name;
            Teacher = teacher;
            _Students = new List<Student>();

            _ListOfCourses.Add(this); // kurzus felvétele az összes kurzus közé
        }

        /// <summary>
        /// Új hallgató felvétele.
        /// </summary>
        /// <param name="student">Hallgató.</param>
        /// <returns></returns>
        public Boolean AddStudent(Student student)
        {
            if (!_Students.Contains(student)) // ha még nincs benne a névsorban
            {
                _Students.Add(student); // vegyük fel
                return true; // jelezzük, hogy sikeres volt a felvétel
            }
            else
                return false;
        }

        /// <summary>
        /// Hallgatók listájának lekérdezése.
        /// </summary>
        /// <param name="teacher">A lekérdező oktató.</param>
        /// <returns></returns>
        public List<Student> GetStudentsList(Teacher teacher) // hallgatók listájának lekérdezése
        {
            if (teacher == Teacher) // ha a kurzus oktatója
                return _Students; // akkor lekérdezheti a hallgatók listáját, és módosíthatja is azt
            else
                return null; // különben egy üres listát kap
        }

        /// <summary>
        /// Hallgatók listájának lekérdezése.
        /// </summary>
        /// <param name="student">A lekérdező hallgató.</param>
        /// <returns></returns>
        public List<Student> GetStudentsList(Student student)
        {
            if (_Students.Contains(student)) // ha hallgatója a kurzusnak
                return new List<Student>(_Students); // akkor megkapja a lista másolatát, így nem tudja módosítani az eredetit
            else
                return null; // különben egy üres listát kap
        }

        // osztályszintű tagok:
        private static List<Course> _ListOfCourses = new List<Course>(); // statikus lista, az összes kurzusnak

        /// <summary>
        /// Kurzusok neveinek lekérdezése.
        /// </summary>
        public static List<String> ListedCourseNames
        {
            get
            {
                return _ListOfCourses.Select(course => course.Name).ToList(); // csak a nevet kérdezzük le a listából, és ezt újabb listává alakítjuk
            }
        }

        /// <summary>
        /// Kurzus lekérdezése név alapján.
        /// </summary>
        /// <param name="name">A kurzus neve.</param>
        /// <returns></returns>
        public static Course GetCourse(String name)
        {
            return _ListOfCourses.Where(couse => couse.Name == name).FirstOrDefault();
            // leválogatjuk azokat, ahol a kurzusnév egyezik, és azokbóél az elsőt (vagy ha nincs ilyen, nullát) adunk vissza
        }
    }
}
