﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;

namespace ELTE.Basics.University
{
    /// <summary>
    /// Egyetemi polgár típusa.
    /// </summary>
    abstract class UniversityCitizen
    {
        protected List<Course> _Courses;

        /// <summary>
        /// Egyetemi polgár nevének lekérdezése.
        /// </summary>
        public String Name { get; protected set; }
        /// <summary>
        /// Egyetemi polgár kódjának lekérdezése.
        /// </summary>
        public String UniversityCode { get; protected set; }

        /// <summary>
        /// Kurzusnevek lekérdezése.
        /// </summary>
        public List<String> CourseNames
        {
            get
            {
                return _Courses.Select(course => course.Name).ToList();
            }
        }

        /// <summary>
        /// Egyetemi polgár létrehozása.
        /// </summary>
        /// <param name="name">Egyetemi polgár neve.</param>
        public UniversityCitizen(String name)
        {
            Name = name;
            UniversityCode = GenerateCode(name);
            _Courses = new List<Course>();

            _ListOfCitizens.Add(this); // aktuálisan mindig egy hallgató, vagy oktató példány lesz
        }

        protected static List<UniversityCitizen> _ListOfCitizens = new List<UniversityCitizen>();

        protected static String GenerateCode(String name) // kódgenerálás, amihez ismerni kell a többi polgár kódját is
        {
            String code = Char.ToUpper(name[0]) + Char.ToUpper(name[1]) + Char.ToUpper(name[name.IndexOf(' ') + 1]) + "PAAT.ELTE"; // kód generálása a név alapján
            Char modifierChar = 'A';

            foreach (Student s in _ListOfCitizens)
            {
                if (code == s.UniversityCode) // ha találtunk ugyanolyan nazonosítójú oktatót
                {
                    modifierChar++; // akkor módosítjuk az egyik karaktert
                    code = Char.ToUpper(name[0]) + Char.ToUpper(name[1]) + Char.ToUpper(name[name.IndexOf(' ') + 1]) + "PA" + modifierChar + "T.ELTE";
                }
            }
            return code;
        }
    }
}
